@@ -15,7 +15,7 @@ extremely handy for recovering the syntactic structure of actual rules, so that'
 import re, os, collections, typing
 from ..support import foundation, compaction, interfaces
 from ..parsing import automata
-from ..scanning import regular, miniscan
+from ..scanning import finite, regular, miniscan
 from . import grammar
 
 
@@ -25,7 +25,7 @@ def compile_file(pathname, *, method, strict=False) -> dict:
 
 class TextBookForm:
 	""" This provides the various views of the text-book form of scan and parse tables. """
-	def __init__(self, *, dfa: regular.DFA, scan_actions:list, parse_table: automata.DragonBookTable):
+	def __init__(self, *, dfa: finite.DFA, scan_actions:list, parse_table: automata.DragonBookTable):
 		self.dfa = dfa
 		self.scan_actions = scan_actions
 		self.parse_table = parse_table
@@ -74,7 +74,7 @@ class TextBookForm:
 			self.parse_table.make_csv(pathstem)
 
 class IntermediateForm(typing.NamedTuple):
-	nfa: regular.NFA
+	nfa: finite.NFA
 	scan_actions: list
 	hfa: automata.HFA
 	parse_style:automata.ParsingStyle
@@ -84,6 +84,13 @@ class IntermediateForm(typing.NamedTuple):
 	def make_dot_file(self, path): self.hfa.make_dot_file(path)
 
 
+STRERROR = {
+	regular.BadReferenceError: "The name {0} is not defined.",
+	regular.CircularDefinitionError: "The name {0} is involved in a circular definition.",
+	regular.NonClassError: "The name {0} is used in a character class, but is not defined as a character (class).",
+	regular.TrailingContextError: "Variable size for both stem and trailing context is not currently supported.",
+}
+
 def compile_string(document:str, *, method) -> IntermediateForm:
 	""" This has the job of reading the specification and building the textbook-form tables. """
 	# The approach is a sort of outside-in parse. The outermost layer concerns the overall markdown document format,
@@ -91,8 +98,8 @@ def compile_string(document:str, *, method) -> IntermediateForm:
 	# Each major sub-language is line-oriented and interpreted with one of the following five subroutines:
 	
 	def handle_meta_exception(e: Exception):
-		if isinstance(e, miniscan.PatternError):
-			raise grammar.DefinitionError('At line %d: %s'%(line_number, e.args[0])) from None
+		if isinstance(e, regular.PatternError):
+			raise grammar.DefinitionError('At line %d: %s'%(line_number, STRERROR[type(e)].format(e.args))) from None
 		elif isinstance(e, interfaces.LanguageError):
 			raise grammar.DefinitionError('At line %d: Malformed pattern.' % line_number) from None
 		else: raise e
@@ -101,7 +108,7 @@ def compile_string(document:str, *, method) -> IntermediateForm:
 		name, regex = current_line_text.split(None, 1)
 		if name in env: raise grammar.DefinitionError('You cannot redefine named subexpression %r at line %d.'%(name, line_number))
 		if not re.fullmatch(r'[A-Za-z][A-Za-z_]+', name): raise grammar.DefinitionError('Subexpression %r ought to obey the rule at line %d.'%(name, line_number))
-		try: env[name] = miniscan.rex.parse(miniscan.META.scan(regex, env=env), language='Regular')
+		try: env[name] = miniscan.rex.parse(miniscan.META.scan(regex), language='Regular')
 		except Exception as e: handle_meta_exception(e)
 		assert isinstance(env[name], regular.Regular), "This would be a bug."
 	
@@ -145,7 +152,7 @@ def compile_string(document:str, *, method) -> IntermediateForm:
 				for q,b in zip(nfa.condition(current_pattern_group), bol):
 					if b: nfa.link_epsilon(q, src)
 				nfa.final[dst] = rule_id
-				expression.encode(src, dst, nfa, rank)
+				regular.Encoder(nfa, rank, env).visit(expression, src, dst)
 		pending_patterns.clear()
 		pass
 	
@@ -185,7 +192,7 @@ def compile_string(document:str, *, method) -> IntermediateForm:
 	
 	# The regular (finite-state) portion of the definition:
 	env = miniscan.PRELOAD['ASCII'].copy()
-	nfa = regular.NFA()
+	nfa = finite.NFA()
 	pending_patterns = collections.defaultdict(list) # Those awaiting an application of the `|` action...
 	scan_actions = [] # That of a regular-language rule entry is <message, parameter, trail, line_number>
 	current_pattern_group = None
