@@ -12,11 +12,14 @@ class definition for a grammar object which supplies the necessary bits to make
 the extensions over BNF work properly.
 """
 
+from typing import List
+
 from ..support import failureprone, interfaces
 from ..scanning import miniscan
 from ..parsing import context_free, miniparse
 
 NONDET = object()
+VOID = object()
 
 class DefinitionError(Exception): pass
 
@@ -35,12 +38,19 @@ class Element:
 	def implement(self, ebnf:"EBNF_Definition", head:str, bindings:dict) -> str:
 		""" Interpreting the MacroParse EBNF variant now seems to require a proper environment... """
 		raise NotImplementedError(type(self))
+	
+	def is_void_for(self, ebnf:"EBNF_Definition", head: str) -> bool:
+		raise NotImplementedError(type(self))
 
 class Symbol(Element):
 	def __init__(self, name): self.name = name
 	
 	def implement(self, ebnf: "EBNF_Definition", head: str, bindings: dict) -> str:
 		return head if self.name == '_' else bindings.get(self.name, self.name)
+	
+	def is_void_for(self, ebnf: "EBNF_Definition", head: str) -> bool:
+		key = head if self.name == '_' else self.name
+		return key in ebnf.void_symbols
 
 
 class InlineRenaming(Element):
@@ -55,7 +65,9 @@ class InlineRenaming(Element):
 			for a in alts:
 				ebnf.plain_cfg.rule(symbol, [a], None, None, 0, None)
 		return symbol
-		
+	
+	def is_void_for(self, ebnf: "EBNF_Definition", head: str) -> bool:
+		return all(a.is_void_for(ebnf, head) for a in self.alternatives)
 
 class MacroCall(Element):
 	def __init__(self, name, actual_parameters):
@@ -69,6 +81,9 @@ class MacroCall(Element):
 		symbol = "%s(%s)"%(self.name, ",".join(args))
 		if ebnf.implementing(symbol): ebnf.must_elaborate.append((symbol, self.name, args))
 		return symbol
+	
+	def is_void_for(self, ebnf: "EBNF_Definition", head: str) -> bool:
+		return self.name in ebnf.void_symbols
 
 
 class Rewrite:
@@ -77,18 +92,18 @@ class Rewrite:
 		self.precsym = precsym
 		self.line_nr = 0
 		self.message:Action = self.elements.pop(-1) if isinstance(self.elements[-1], Action) else None
-		self.__args = args = []
 		self.size = len(self.elements)
-		for i,elt in enumerate(self.elements):
-			if hasattr(elt, 'capture'): args.append(i)
-		if not args: args.extend(range(self.size)) # Pick up everything if nothing is special.
 	
-	def prefix_capture(self, size:int):
-		return tuple(c - size for c in self.__args if c < size)
 	
 	def install(self, ebnf:"EBNF_Definition", head, bindings):
+		# Work out which places on the stack (relative to the left edge of the rule)
+		# contain "significant" data (for the action rule):
+		args = [i for i,elt in enumerate(self.elements) if hasattr(elt, 'capture')]
+		if not args: # If no RHS elements specifically have capture marks, respect the void-symbols set.
+			args = [i for i,elt in enumerate(self.elements) if not elt.is_void_for(ebnf, head)]
+		
+		
 		ebnf.error_help.current_line_nr = self.line_nr # Because MACROS.
-		rhs = [elt.implement(ebnf, head, bindings) for elt in self.elements]
 		"""
 		Install one rewrite rule:
 		Does everything necessary to interpret extension forms down to plain BNF,
@@ -99,18 +114,25 @@ class Rewrite:
 			if isinstance(elt, Action):
 				placeholder = ':'+elt.name
 				raw_bnf.append(placeholder)
-				ebnf.internal_action(placeholder, self.prefix_capture(i))
+				ebnf.internal_action(placeholder, prefix_capture(args, i))
 			else:
 				assert isinstance(elt, Element)
 				raw_bnf.append(elt.implement(ebnf, head, bindings))
 		
 		if self.message is None:
 			con = None
-			plc = self.__args[0] if len(self.__args) == 1 else self.__args
+			plc = args[0] if len(args) == 1 else args
 		else:
-			con, plc = self.message.name, self.__args
+			con, plc = self.message.name, args
 		ebnf.plain_cfg.rule(head, raw_bnf, self.precsym, con, plc, ebnf.error_help.current_line_nr)
 
+def prefix_capture(args:List[int], size: int):
+	# This is returning offsets from the size of the stack as seen
+	# by an intermediate action (which is really a special kind of
+	# epsilon-rule).
+	return tuple(c - size for c in args if c < size)
+
+
 """
 The MacroParse metagrammar contains various repetition constructs. The following functions
 act as rudimentary macros to relieve some redundancy; they essentially mean something like:
@@ -138,21 +160,22 @@ def APPEND(the_list, element):
 
 ### The MacroParse metagrammar for individual production lines is as follows.
 METAGRAMMAR = miniparse.MiniParse('production', 'precedence', 'condition')
+METAGRAMMAR.void_symbols.update('arrow capture ( ) [ ] , pragma_prec pragma_nondeterministic'.split())
 
-METAGRAMMAR.rule('production', '.head .' + list_of('rewrite', '|'))(None)
+METAGRAMMAR.rule('production', 'head ' + list_of('rewrite', '|'))(None)
 
 METAGRAMMAR.rule('head', '|')(None) # "use prior" is represented by a null "head".
-METAGRAMMAR.rule('head', '.name arrow', )(None) # This will come across as a string.
-METAGRAMMAR.rule('head', '.name ( .%s ) arrow' % list_of('name', ','))(None) # Represent macro heads with (name, args) tuples.
+METAGRAMMAR.rule('head', 'name arrow', )(None) # This will come across as a string.
+METAGRAMMAR.rule('head', 'name ( %s ) arrow' % list_of('name', ','))(None) # Represent macro heads with (name, args) tuples.
 
 ELEMENTS = one_or_more('element')
 METAGRAMMAR.rule('rewrite', ELEMENTS)(Rewrite)
-METAGRAMMAR.rule('rewrite', '.' + ELEMENTS + ' pragma_prec .terminal')(Rewrite)
+METAGRAMMAR.rule('rewrite', ELEMENTS + ' pragma_prec terminal')(Rewrite)
 
 METAGRAMMAR.renaming('terminal', 'name', 'literal')
 
 METAGRAMMAR.renaming('element', 'positional_element')
-@METAGRAMMAR.rule('element', 'capture .positional_element')
+@METAGRAMMAR.rule('element', 'capture positional_element')
 def _capture_(positional_element:Element) -> Element:
 	positional_element.capture = True
 	return positional_element
@@ -162,26 +185,27 @@ METAGRAMMAR.rule('positional_element', 'message')(Action)
 METAGRAMMAR.rule('positional_element', 'err_token')(lambda x:Symbol(interfaces.ERROR_SYMBOL))
 
 METAGRAMMAR.renaming('actual_parameter', 'symbol') # alternation brackets should not nest directly...
-METAGRAMMAR.rule('actual_parameter', '[ .' + one_or_more('symbol') + ' ]')(InlineRenaming)
+METAGRAMMAR.rule('actual_parameter', '[ ' + one_or_more('symbol') + ' ]')(InlineRenaming)
 
 METAGRAMMAR.rule('symbol', 'name')(Symbol)    # Normal symbol
 METAGRAMMAR.rule('symbol', 'literal')(Symbol) # Also a normal symbol with a funny name
 METAGRAMMAR.rule('symbol', 'topic')(Symbol)   # Topic symbol; gets its name fixed later.
-METAGRAMMAR.rule('symbol', '.name ( .' + list_of('actual_parameter', ',') + ' )')(MacroCall)
+METAGRAMMAR.rule('symbol', 'name ( ' + list_of('actual_parameter', ',') + ' )')(MacroCall)
 
 # Sub-language: For specifying precedence and associativity rules:
 NAMES = one_or_more('name')
-METAGRAMMAR.rule('precedence', '.associativity .'+one_or_more('terminal'))(None)
-METAGRAMMAR.rule('precedence', 'pragma_nondeterministic .'+NAMES)(lambda x:(NONDET, x))
+METAGRAMMAR.rule('precedence', 'associativity '+one_or_more('terminal'))(None)
+METAGRAMMAR.rule('precedence', 'pragma_nondeterministic '+NAMES)(lambda x:(NONDET, x))
 
 METAGRAMMAR.rule('associativity', 'pragma_left')(lambda x: context_free.LEFT)
 METAGRAMMAR.rule('associativity', 'pragma_right')(lambda x: context_free.RIGHT)
 METAGRAMMAR.rule('associativity', 'pragma_nonassoc')(lambda x: context_free.NONASSOC)
 METAGRAMMAR.rule('associativity', 'pragma_bogus')(lambda x: context_free.BOGUS)
+METAGRAMMAR.rule('associativity', 'pragma_void')(lambda x: VOID)
 
 # Sub-language: For specifying the connections between scan conditions:
 METAGRAMMAR.rule('condition', 'name')(lambda x:(x,[]))
-METAGRAMMAR.rule('condition', '.name arrow .'+NAMES)(None)
+METAGRAMMAR.rule('condition', 'name arrow '+NAMES)(None)
 
 ### The lexeme definitions for production rule lines are as follows:
 LEX = miniscan.Definition()
@@ -213,7 +237,7 @@ class ErrorHelper:
 		self.current_line_nr = line_nr
 		metascan = LEX.scan(line)
 		try: return METAGRAMMAR.parse(metascan, language=language)
-		except interfaces.ScanError as e: self.gripe_about(line, e.args[0], "The MacroParse MetaScanner got confused by %r" % e.args[1])
+		except interfaces.ScannerBlocked as e: self.gripe_about(line, e.args[0], "The MacroParse MetaScanner got confused by %r" % e.args[1])
 		except interfaces.ParseError as e:
 			self.gripe_about(
 				line, metascan.current_position(),
@@ -251,10 +275,12 @@ class EBNF_Definition:
 		self.must_elaborate = []
 		self.error_help = error_help
 		self.nondeterministic_symbols = set()
+		self.void_symbols = set()
 	
 	def read_precedence_line(self, line:str, line_nr:int):
 		direction, symbols = self.error_help.parse(line, line_nr, 'precedence')
 		if direction is NONDET: self.nondeterministic_symbols.update(symbols)
+		elif direction is VOID: self.void_symbols.update(symbols)
 		else: self.plain_cfg.assoc(direction, symbols)
 
 	def read_production_line(self, line:str, line_nr:int):
